@@ -16,7 +16,7 @@
 #' @examples
 #' \dontrun{
 #'## Load a dataset from the mvabund package
-#'data(antTraits)
+#'data(antTraits, package = "mvabund")
 #'y <- as.matrix(antTraits$abund)
 #'X <- as.matrix(antTraits$env)
 #'TR <- antTraits$traits
@@ -32,30 +32,25 @@
 #'@export randomCoefplot.gllvm
 randomCoefplot.gllvm <- function(object, y.label = TRUE, which.Xcoef = NULL, cex.ylab = 0.5, mfrow = NULL, mar = c(4,6,2,1), xlim.list = NULL, order = FALSE, ...)
 {
-  
+  if(is.null(object$lv.X.design) && !is.null(object$lv.X))object$lv.X.design <- object$lv.X # backward compatibility
   if (any(class(object) != "gllvm"))
     stop("Class of the object isn't 'gllvm'.")
-  
-  if ((is.null(object$Xrandom) || is.null(object$randomX)) && isFALSE(object$randomB))
+  if(is.null(object$col.eff$col.eff))object$col.eff$col.eff <- FALSE # backward compatibility
+  if ((is.null(object$Xrandom) || is.null(object$randomX)) && isFALSE(object$randomB)  && object$col.eff$col.eff!="random")
     stop("No random covariates in the model.")
+  if(is.null(object$TR) && object$col.eff$col.eff=="random") object$Xr <- as.matrix(object$col.eff$spdr)
   
-  if((object$num.lv.c+object$num.RR)==0){
+  if((object$num.lv.c+object$num.RR)==0 && !is.null(object$params$Br)){
     if(is.null(which.Xcoef))which.Xcoef <- c(1:NROW(object$params$Br))
     Xcoef <- as.matrix(t(object$params$Br)[,which.Xcoef,drop=F])
-    cnames <- colnames(object$Xr[,which.Xcoef])
+    cnames <- colnames(object$Xr[,which.Xcoef,drop=FALSE])
     k <- length(cnames)
     if(is.null(colnames(object$y))) 
       colnames(object$y) <- paste("Y",1:NCOL(object$y), sep = "")
     m <- ncol(object$y)
     Xc <- Xcoef
     
-    if((object$method %in% c("VA", "EVA"))){
-      object$Ab <- object$Ab+CMSEPf(object)$Ab
-      # object$Ab <- object$Ab+sdB(object)
-      sdXcoef <- t(sqrt(apply(object$Ab,1,diag)))
-    } else {
-      sdXcoef <- t(sqrt(apply(object$prediction.errors$Br,1,diag)))
-    }
+    sdXcoef <- t(getPredictErr(object)$Br)
     sdXcoef <- sdXcoef[,which.Xcoef,drop=F]
     if (is.null(mfrow) && k > 1)
       mfrow <- c(1, k)
@@ -87,17 +82,39 @@ randomCoefplot.gllvm <- function(object, y.label = TRUE, which.Xcoef = NULL, cex
       if (y.label)
         axis( 2, at = At.y, labels = names(Xc), las = 1, cex.axis = cex.ylab)
     }
-  }else{
-    if(is.null(which.Xcoef))which.Xcoef <- c(1:NROW(object$params$LvXcoef))
-    Xcoef <- as.matrix(object$params$theta[,1:(object$num.RR+object$num.lv.c),drop=F]%*%t(object$params$LvXcoef))[,which.Xcoef,drop=F]
-    cnames <- colnames(object$lv.X[,which.Xcoef,drop=F])
+  }
+  
+  if((object$num.lv.c+object$num.RR)>0 || object$col.eff$col.eff == "random"){
+
+    if((object$num.lv.c+object$num.RR)>0 && object$col.eff$col.eff == "random"){
+      if(is.null(which.Xcoef))which.Xcoef <- c(colnames(object$lv.X.design), colnames(object$col.eff$spdr))
+    } else if((object$num.lv.c+object$num.RR)>0){
+      if(is.null(which.Xcoef))which.Xcoef <- colnames(object$lv.X.design)
+    }else if(object$col.eff$col.eff=="random"){
+      if(is.null(which.Xcoef))which.Xcoef <- colnames(object$col.eff$spdr)
+    }
+    Xcoef <- cnames <- sdXcoef <- NULL
+    
+    if((object$num.lv.c+object$num.RR)>0 && any(which.Xcoef %in% colnames(object$lv.X.design))){
+      if(!is.null(object$lv.X) && is.null(object$lv.X.design))object$lv.X.design <- object$lv.X #for backward compatibility
+      Xcoef <- cbind(Xcoef,as.matrix(object$params$theta[,1:(object$num.RR+object$num.lv.c),drop=F]%*%t(object$params$LvXcoef))[,colnames(object$lv.X.design)%in%which.Xcoef,drop=F])
+      cnames <- c(cnames, colnames(object$lv.X.design[,colnames(object$lv.X.design)%in%which.Xcoef,drop=F]))
+      sdXcoef <- cbind(sdXcoef, RRse(object)[,colnames(object$lv.X.design)%in%which.Xcoef,drop=F])
+    }
+    if(object$col.eff$col.eff=="random"){
+      Xcoef <- cbind(Xcoef, t(object$params$Br)[,colnames(object$col.eff$spdr)%in%which.Xcoef,drop=F])
+      cnames <- c(cnames, row.names(object$params$Br))
+      sdXcoef <- cbind(sdXcoef, t(getPredictErr(object)$Br)) 
+    }
+  }
+
+  if((object$num.lv.c+object$num.RR)>0 | object$col.eff$col.eff=="random"){
     k <- length(cnames)
     if(is.null(colnames(object$y))) 
       colnames(object$y) <- paste("Y",1:NCOL(object$y), sep = "")
     labely <- colnames(object$y)
     m <- length(labely)
     Xc <- Xcoef
-    sdXcoef <- RRse(object)[,which.Xcoef,drop=F]
     
     if (is.null(mfrow) && k > 1)
       mfrow <- c(1, k)
